@@ -2,19 +2,7 @@
  * @file decode.c
  *
  * @section LICENSE
- * Copyright 2012 - 2016
- *
- * Mathis Rosenhauer, Moritz Hanke, Joerg Behrens
- * Deutsches Klimarechenzentrum GmbH
- * Bundesstr. 45a
- * 20146 Hamburg Germany
- *
- * Luis Kornblueh
- * Max-Planck-Institut fuer Meteorologie
- * Bundesstr. 53
- * 20146 Hamburg
- * Germany
- *
+ * Copyright 2026 Mathis Rosenhauer, Moritz Hanke, Joerg Behrens, Luis Kornblueh
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -44,23 +32,23 @@
  * @section DESCRIPTION
  *
  * Adaptive Entropy Decoder
- * Based on CCSDS documents 121.0-B-2 and 120.0-G-3
+ * Based on the CCSDS recommended standard 121.0-B-3
  *
  */
 
+#include "config.h"
+#include "decode.h"
+#include "libaec.h"
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 
-#include "libaec.h"
-#include "decode.h"
-
-#if HAVE_BSR64
-#  include <intrin.h>
+#ifdef HAVE_BSR64
+#include <intrin.h>
 #endif
 
 #define ROS 5
-#define RSI_USED(state) ((size_t)(state->rsip - state->rsi_buffer))
+#define RSI_USED_SIZE(state) ((size_t)(state->rsip - state->rsi_buffer))
 #define BUFFERSPACE(strm) (strm->avail_in >= strm->state->in_blklen      \
                            && strm->avail_out >= strm->state->out_blklen)
 
@@ -68,7 +56,7 @@
     static void flush_##KIND(struct aec_stream *strm)                    \
     {                                                                    \
         uint32_t *flush_end, *bp, half_d;                                \
-        int32_t data, m;                                                 \
+        uint32_t xmax, d, data, m;                                       \
         struct internal_state *state = strm->state;                      \
                                                                          \
         flush_end = state->rsip;                                         \
@@ -87,49 +75,47 @@
             }                                                            \
                                                                          \
             data = state->last_out;                                      \
+            xmax = state->xmax;                                          \
                                                                          \
             if (state->xmin == 0) {                                      \
-                uint32_t xmax, med, d;                                   \
+                uint32_t med;                                            \
                 med = state->xmax / 2 + 1;                               \
-                xmax = state->xmax;                                      \
                                                                          \
                 for (bp = state->flush_start; bp < flush_end; bp++) {    \
+                    uint32_t mask;                                       \
                     d = *bp;                                             \
                     half_d = (d >> 1) + (d & 1);                         \
                     /*in this case: data >= med == data & med */         \
-                    uint32_t mask = (data & med)?xmax:0;                 \
+                    mask = (data & med)?xmax:0;                          \
                                                                          \
                     /*in this case: xmax - data == xmax ^ data */        \
-                    if (half_d <= (mask ^ (uint32_t)data)) {             \
+                    if (half_d <= (mask ^ data)) {                       \
                         data += (d >> 1)^(~((d & 1) - 1));               \
                     } else {                                             \
                         data = mask ^ d;                                 \
                     }                                                    \
-                    put_##KIND(strm, (uint32_t)data);                    \
+                    put_##KIND(strm, data);                              \
                 }                                                        \
                 state->last_out = data;                                  \
             } else {                                                     \
-                int32_t xmax, d;                                         \
-                xmax = state->xmax;                                      \
-                                                                         \
                 for (bp = state->flush_start; bp < flush_end; bp++) {    \
                     d = *bp;                                             \
-                    half_d = ((uint32_t)d >> 1) + (d & 1);               \
+                    half_d = (d >> 1) + (d & 1);                         \
                                                                          \
-                    if (data < 0) {                                      \
-                        if (half_d <= xmax + (uint32_t)data + 1) {       \
-                            data += ((uint32_t)d >> 1)^(~((d & 1) - 1)); \
+                    if ((int32_t)data < 0) {                             \
+                        if (half_d <= xmax + data + 1) {                 \
+                            data += (d >> 1)^(~((d & 1) - 1));           \
                         } else {                                         \
                             data = d - xmax - 1;                         \
                         }                                                \
                     } else {                                             \
-                        if (half_d <= xmax - (uint32_t)data) {           \
-                            data += ((uint32_t)d >> 1)^(~((d & 1) - 1)); \
+                        if (half_d <= xmax - data) {                     \
+                            data += (d >> 1)^(~((d & 1) - 1));           \
                         } else {                                         \
                             data = xmax - d;                             \
                         }                                                \
                     }                                                    \
-                    put_##KIND(strm, (uint32_t)data);                    \
+                    put_##KIND(strm, data);                              \
                 }                                                        \
                 state->last_out = data;                                  \
             }                                                            \
@@ -196,27 +182,12 @@ FLUSH(lsb_24)
 FLUSH(lsb_16)
 FLUSH(8)
 
-static inline void check_rsi_end(struct aec_stream *strm)
-{
-    /**
-       Flush output if end of RSI reached
-     */
-    struct internal_state *state = strm->state;
-
-    if (state->rsi_size == RSI_USED(state)) {
-        state->flush_output(strm);
-        state->flush_start = state->rsi_buffer;
-        state->rsip = state->rsi_buffer;
-    }
-}
-
 static inline void put_sample(struct aec_stream *strm, uint32_t s)
 {
     struct internal_state *state = strm->state;
 
     *state->rsip++ = s;
     strm->avail_out -= state->bytes_per_sample;
-    check_rsi_end(strm);
 }
 
 static inline uint32_t direct_get(struct aec_stream *strm, int n)
@@ -228,11 +199,9 @@ static inline uint32_t direct_get(struct aec_stream *strm, int n)
      */
 
     struct internal_state *state = strm->state;
-    int b;
-
     if (state->bitp < n)
     {
-        b = (63 - state->bitp) >> 3;
+        int b = (63 - state->bitp) >> 3;
         if (b == 6) {
             state->acc = (state->acc << 48)
                 | ((uint64_t)strm->next_in[0] << 40)
@@ -295,11 +264,6 @@ static inline uint32_t direct_get_fs(struct aec_stream *strm)
      */
 
     uint32_t fs = 0;
-#if HAVE_BSR64
-    unsigned long i;
-#else
-    int i;
-#endif
     struct internal_state *state = strm->state;
 
     if (state->bitp)
@@ -308,6 +272,9 @@ static inline uint32_t direct_get_fs(struct aec_stream *strm)
         state->acc = 0;
 
     while (state->acc == 0) {
+        if (strm->avail_in < 7)
+            return 0;
+
         state->acc = (state->acc << 56)
             | ((uint64_t)strm->next_in[0] << 48)
             | ((uint64_t)strm->next_in[1] << 40)
@@ -322,17 +289,23 @@ static inline uint32_t direct_get_fs(struct aec_stream *strm)
         state->bitp = 56;
     }
 
-#if HAVE_DECL___BUILTIN_CLZLL
-    i = 63 - __builtin_clzll(state->acc);
-#elif HAVE_BSR64
-    _BitScanReverse64(&i, state->acc);
-#else
-    i = state->bitp - 1;
-    while ((state->acc & (UINT64_C(1) << i)) == 0)
-        i--;
+    {
+#ifndef __has_builtin
+#define __has_builtin(x) 0  /* Compatibility with non-clang compilers. */
 #endif
-    fs += state->bitp - i - 1;
-    state->bitp = i;
+#if HAVE_DECL___BUILTIN_CLZLL || __has_builtin(__builtin_clzll)
+        int i = 63 - __builtin_clzll(state->acc);
+#elif defined HAVE_BSR64
+        unsigned long i;
+        _BitScanReverse64(&i, state->acc);
+#else
+        int i = state->bitp - 1;
+        while ((state->acc & (UINT64_C(1) << i)) == 0)
+            i--;
+#endif
+        fs += state->bitp - i - 1;
+        state->bitp = i;
+    }
     return fs;
 }
 
@@ -396,25 +369,47 @@ static inline uint32_t copysample(struct aec_stream *strm)
     return 1;
 }
 
-static int m_id(struct aec_stream *strm)
+static inline int m_id(struct aec_stream *strm)
+{
+    struct internal_state *state = strm->state;
+    if (strm->avail_in >= strm->state->in_blklen) {
+        state->id = direct_get(strm, state->id_len);
+    } else {
+        if (bits_ask(strm, state->id_len) == 0) {
+            state->mode = m_id;
+            return M_EXIT;
+        }
+        state->id = bits_get(strm, state->id_len);
+        bits_drop(strm, state->id_len);
+    }
+    state->mode = state->id_table[state->id];
+    return(state->mode(strm));
+}
+
+static int m_next_cds(struct aec_stream *strm)
 {
     struct internal_state *state = strm->state;
 
-    if (state->rsip == state->rsi_buffer) {
-        if(strm->flags & AEC_PAD_RSI)
-            state->bitp -= state->bitp % 8;
-        if (state->pp)
+    if ((state->offsets != NULL) && (state->rsi_size == RSI_USED_SIZE(state)))
+        vector_push_back(
+            state->offsets,
+            strm->total_in * 8 - (strm->avail_in * 8 + state->bitp));
+
+    if (state->rsi_size == RSI_USED_SIZE(state)) {
+        state->flush_output(strm);
+        state->flush_start = state->rsi_buffer;
+        state->rsip = state->rsi_buffer;
+        if (state->pp) {
             state->ref = 1;
+            state->encoded_block_size = strm->block_size - 1;
+        }
+        if (strm->flags & AEC_PAD_RSI)
+            state->bitp -= state->bitp % 8;
     } else {
         state->ref = 0;
+        state->encoded_block_size = strm->block_size;
     }
-    if (bits_ask(strm, state->id_len) == 0)
-        return M_EXIT;
-    state->id = bits_get(strm, state->id_len);
-    bits_drop(strm, state->id_len);
-    state->mode = state->id_table[state->id];
-
-    return M_CONTINUE;
+    return m_id(strm);
 }
 
 static int m_split_output(struct aec_stream *strm)
@@ -431,10 +426,9 @@ static int m_split_output(struct aec_stream *strm)
             state->rsip++;
         strm->avail_out -= state->bytes_per_sample;
         bits_drop(strm, k);
-    } while(++state->i < state->n);
+    } while(++state->sample_counter < state->encoded_block_size);
 
-    check_rsi_end(strm);
-    state->mode = m_id;
+    state->mode = m_next_cds;
     return M_CONTINUE;
 }
 
@@ -446,11 +440,11 @@ static int m_split_fs(struct aec_stream *strm)
     do {
         if (fs_ask(strm) == 0)
             return M_EXIT;
-        state->rsip[state->i] = state->fs << k;
+        state->rsip[state->sample_counter] = state->fs << k;
         fs_drop(strm);
-    } while(++state->i < state->n);
+    } while(++state->sample_counter < state->encoded_block_size);
 
-    state->i = 0;
+    state->sample_counter = 0;
     state->mode = m_split_output;
 
     return M_CONTINUE;
@@ -458,43 +452,36 @@ static int m_split_fs(struct aec_stream *strm)
 
 static int m_split(struct aec_stream *strm)
 {
-    size_t i;
-    int k;
     struct internal_state *state = strm->state;
 
     if (BUFFERSPACE(strm)) {
-        k = state->id - 1;
+        int k = state->id - 1;
+        size_t binary_part = (k * state->encoded_block_size) / 8 + 9;
 
         if (state->ref)
             *state->rsip++ = direct_get(strm, strm->bits_per_sample);
 
-        for (i = 0; i < strm->block_size - state->ref; i++)
+        for (size_t i = 0; i < state->encoded_block_size; i++)
             state->rsip[i] = direct_get_fs(strm) << k;
 
         if (k) {
-            for (i = state->ref; i < strm->block_size; i++)
+            if (strm->avail_in < binary_part)
+                return M_ERROR;
+
+            for (size_t i = 0; i < state->encoded_block_size; i++)
                 *state->rsip++ += direct_get(strm, k);
         } else {
-            state->rsip += strm->block_size - state->ref;
+            state->rsip += state->encoded_block_size;
         }
 
         strm->avail_out -= state->out_blklen;
-        check_rsi_end(strm);
-
-        state->mode = m_id;
-        return M_CONTINUE;
-    }
-
-    if (state->ref) {
-        if (copysample(strm) == 0)
-            return M_EXIT;
-        state->n = strm->block_size - 1;
+        state->mode = m_next_cds;
     } else {
-        state->n = strm->block_size;
+        if (state->ref && (copysample(strm) == 0))
+            return M_EXIT;
+        state->sample_counter = 0;
+        state->mode = m_split_fs;
     }
-
-    state->i = 0;
-    state->mode = m_split_fs;
     return M_CONTINUE;
 }
 
@@ -506,94 +493,95 @@ static int m_zero_output(struct aec_stream *strm)
         if (strm->avail_out < state->bytes_per_sample)
             return M_EXIT;
         put_sample(strm, 0);
-    } while(--state->i);
+    } while(--state->sample_counter);
 
-    state->mode = m_id;
+    state->mode = m_next_cds;
     return M_CONTINUE;
 }
 
 static int m_zero_block(struct aec_stream *strm)
 {
-    uint32_t i, zero_blocks, b, zero_bytes;
     struct internal_state *state = strm->state;
+    uint32_t zero_blocks;
+    uint32_t zero_samples;
+    uint32_t zero_bytes;
 
     if (fs_ask(strm) == 0)
         return M_EXIT;
+
     zero_blocks = state->fs + 1;
     fs_drop(strm);
 
     if (zero_blocks == ROS) {
-        b = (int)RSI_USED(state) / strm->block_size;
-        zero_blocks = MIN(strm->rsi - b, 64 - (b % 64));
+        int b = (int)RSI_USED_SIZE(state) / strm->block_size;
+        zero_blocks = MIN((int)(strm->rsi - b), 64 - (b % 64));
     } else if (zero_blocks > ROS) {
         zero_blocks--;
     }
 
-    if (state->ref)
-        i = zero_blocks * strm->block_size - 1;
-    else
-        i = zero_blocks * strm->block_size;
-
-    zero_bytes = i * state->bytes_per_sample;
+    zero_samples = zero_blocks * strm->block_size - state->ref;
+    if (state->rsi_size - RSI_USED_SIZE(state) < zero_samples)
+        return M_ERROR;
 
+    zero_bytes = zero_samples * state->bytes_per_sample;
     if (strm->avail_out >= zero_bytes) {
-        if (state->rsi_size - RSI_USED(state) < i)
-            return M_ERROR;
-
-        memset(state->rsip, 0, i * sizeof(uint32_t));
-        state->rsip += i;
+        memset(state->rsip, 0, zero_samples * sizeof(uint32_t));
+        state->rsip += zero_samples;
         strm->avail_out -= zero_bytes;
-        check_rsi_end(strm);
-
-        state->mode = m_id;
-        return M_CONTINUE;
+        state->mode = m_next_cds;
+    } else {
+        state->sample_counter = zero_samples;
+        state->mode = m_zero_output;
     }
-
-    state->i = i;
-    state->mode = m_zero_output;
     return M_CONTINUE;
 }
 
 static int m_se_decode(struct aec_stream *strm)
 {
-    int32_t m, d1;
     struct internal_state *state = strm->state;
 
-    while(state->i < strm->block_size) {
+    while(state->sample_counter < strm->block_size) {
+        int32_t m;
+        int32_t d1;
         if (fs_ask(strm) == 0)
             return M_EXIT;
         m = state->fs;
+        if (m > SE_TABLE_SIZE)
+            return M_ERROR;
         d1 = m - state->se_table[2 * m + 1];
 
-        if ((state->i & 1) == 0) {
+        if ((state->sample_counter & 1) == 0) {
             if (strm->avail_out < state->bytes_per_sample)
                 return M_EXIT;
             put_sample(strm, state->se_table[2 * m] - d1);
-            state->i++;
+            state->sample_counter++;
         }
 
         if (strm->avail_out < state->bytes_per_sample)
             return M_EXIT;
         put_sample(strm, d1);
-        state->i++;
+        state->sample_counter++;
         fs_drop(strm);
     }
 
-    state->mode = m_id;
+    state->mode = m_next_cds;
     return M_CONTINUE;
 }
 
 static int m_se(struct aec_stream *strm)
 {
-    uint32_t i;
-    int32_t m, d1;
     struct internal_state *state = strm->state;
 
     if (BUFFERSPACE(strm)) {
-        i = state->ref;
+        uint32_t i = state->ref;
 
         while (i < strm->block_size) {
-            m = direct_get_fs(strm);
+            uint32_t m = direct_get_fs(strm);
+            int32_t d1;
+
+            if (m > SE_TABLE_SIZE)
+                return M_ERROR;
+
             d1 = m - state->se_table[2 * m + 1];
 
             if ((i & 1) == 0) {
@@ -603,12 +591,11 @@ static int m_se(struct aec_stream *strm)
             put_sample(strm, d1);
             i++;
         }
-        state->mode = m_id;
-        return M_CONTINUE;
+        state->mode = m_next_cds;
+    } else {
+        state->mode = m_se_decode;
+        state->sample_counter = state->ref;
     }
-
-    state->mode = m_se_decode;
-    state->i = state->ref;
     return M_CONTINUE;
 }
 
@@ -619,12 +606,10 @@ static int m_low_entropy_ref(struct aec_stream *strm)
     if (state->ref && copysample(strm) == 0)
         return M_EXIT;
 
-    if(state->id == 1) {
+    if (state->id == 1)
         state->mode = m_se;
-        return M_CONTINUE;
-    }
-
-    state->mode = m_zero_block;
+    else
+        state->mode = m_zero_block;
     return M_CONTINUE;
 }
 
@@ -647,40 +632,34 @@ static int m_uncomp_copy(struct aec_stream *strm)
     do {
         if (copysample(strm) == 0)
             return M_EXIT;
-    } while(--state->i);
+    } while(--state->sample_counter);
 
-    state->mode = m_id;
+    state->mode = m_next_cds;
     return M_CONTINUE;
 }
 
 static int m_uncomp(struct aec_stream *strm)
 {
-    size_t i;
     struct internal_state *state = strm->state;
 
     if (BUFFERSPACE(strm)) {
-        for (i = 0; i < strm->block_size; i++)
+        for (size_t i = 0; i < strm->block_size; i++)
             *state->rsip++ = direct_get(strm, strm->bits_per_sample);
         strm->avail_out -= state->out_blklen;
-        check_rsi_end(strm);
-
-        state->mode = m_id;
-        return M_CONTINUE;
+        state->mode = m_next_cds;
+    } else {
+        state->sample_counter = strm->block_size;
+        state->mode = m_uncomp_copy;
     }
-
-    state->i = strm->block_size;
-    state->mode = m_uncomp_copy;
     return M_CONTINUE;
 }
 
 static void create_se_table(int *table)
 {
-    int i, j, k, ms;
-
-    k = 0;
-    for (i = 0; i < 13; i++) {
-        ms = k;
-        for (j = 0; j <= i; j++) {
+    int k = 0;
+    for (int i = 0; i < 13; i++) {
+        int ms = k;
+        for (int j = 0; j <= i; j++) {
             table[2 * k] = i;
             table[2 * k + 1] = ms;
             k++;
@@ -690,8 +669,8 @@ static void create_se_table(int *table)
 
 int aec_decode_init(struct aec_stream *strm)
 {
-    int i, modi;
     struct internal_state *state;
+    int modi;
 
     if (strm->bits_per_sample > 32 || strm->bits_per_sample == 0)
         return AEC_CONF_ERROR;
@@ -721,8 +700,7 @@ int aec_decode_init(struct aec_stream *strm)
             else
                 state->flush_output = flush_lsb_32;
         }
-        state->out_blklen = strm->block_size
-            * state->bytes_per_sample;
+        state->out_blklen = strm->block_size * state->bytes_per_sample;
     }
     else if (strm->bits_per_sample > 8) {
         state->bytes_per_sample = 2;
@@ -752,15 +730,15 @@ int aec_decode_init(struct aec_stream *strm)
     }
 
     if (strm->flags & AEC_DATA_SIGNED) {
-        state->xmax = UINT32_MAX >> (32 - strm->bits_per_sample + 1);
+        state->xmax = (INT64_C(1) << (strm->bits_per_sample - 1)) - 1;
         state->xmin = ~state->xmax;
     } else {
         state->xmin = 0;
-        state->xmax = UINT32_MAX >> (32 - strm->bits_per_sample);
+        state->xmax = (UINT64_C(1) << strm->bits_per_sample) - 1;
     }
 
     state->in_blklen = (strm->block_size * strm->bits_per_sample
-                        + state->id_len) / 8 + 9;
+                        + state->id_len) / 8 + 16;
 
     modi = 1UL << state->id_len;
     state->id_table = malloc(modi * sizeof(int (*)(struct aec_stream *)));
@@ -768,7 +746,7 @@ int aec_decode_init(struct aec_stream *strm)
         return AEC_MEM_ERROR;
 
     state->id_table[0] = m_low_entropy;
-    for (i = 1; i < modi - 1; i++) {
+    for (int i = 1; i < modi - 1; i++) {
         state->id_table[i] = m_split;
     }
     state->id_table[modi - 1] = m_uncomp;
@@ -778,7 +756,14 @@ int aec_decode_init(struct aec_stream *strm)
     if (state->rsi_buffer == NULL)
         return AEC_MEM_ERROR;
 
-    state->ref = 0;
+    state->pp = strm->flags & AEC_DATA_PREPROCESS;
+    if (state->pp) {
+        state->ref = 1;
+        state->encoded_block_size = strm->block_size - 1;
+    } else {
+        state->ref = 0;
+        state->encoded_block_size = strm->block_size;
+    }
     strm->total_in = 0;
     strm->total_out = 0;
 
@@ -786,8 +771,9 @@ int aec_decode_init(struct aec_stream *strm)
     state->flush_start = state->rsi_buffer;
     state->bitp = 0;
     state->fs = 0;
-    state->pp = strm->flags & AEC_DATA_PREPROCESS;
     state->mode = m_id;
+    state->offsets = NULL;
+
     return AEC_OK;
 }
 
@@ -830,18 +816,19 @@ int aec_decode(struct aec_stream *strm, int flush)
 int aec_decode_end(struct aec_stream *strm)
 {
     struct internal_state *state = strm->state;
+    if (state->offsets != NULL)
+        vector_destroy(state->offsets);
 
     free(state->id_table);
     free(state->rsi_buffer);
     free(state);
+
     return AEC_OK;
 }
 
 int aec_buffer_decode(struct aec_stream *strm)
 {
-    int status;
-
-    status = aec_decode_init(strm);
+    int status = aec_decode_init(strm);
     if (status != AEC_OK)
         return status;
 
@@ -849,3 +836,118 @@ int aec_buffer_decode(struct aec_stream *strm)
     aec_decode_end(strm);
     return status;
 }
+
+int aec_buffer_seek(struct aec_stream *strm, size_t offset)
+{
+    struct internal_state *state = strm->state;
+
+    size_t byte_offset = offset / 8;
+    unsigned char bit_offset = offset % 8;
+
+    if (strm->avail_in < byte_offset)
+        return AEC_MEM_ERROR;
+
+    strm->next_in += byte_offset;
+    strm->avail_in -= byte_offset;
+
+    if (bit_offset > 0) {
+        if (strm->avail_in < 1)
+            return AEC_MEM_ERROR;
+
+        state->acc = (uint64_t)strm->next_in[0];
+        state->bitp = 8 - bit_offset;
+        strm->next_in++;
+        strm->avail_in--;
+    }
+    return AEC_OK;
+}
+
+int aec_decode_range(struct aec_stream *strm, const size_t *rsi_offsets, size_t rsi_offsets_count, size_t pos, size_t size)
+{
+    struct internal_state *state = strm->state;
+    int status;
+    size_t rsi_size;
+    size_t rsi_n;
+    unsigned char *out_tmp;
+    struct aec_stream strm_tmp = *strm;
+
+    if (state->pp) {
+        state->ref = 1;
+        state->encoded_block_size = strm->block_size - 1;
+    } else {
+        state->ref = 0;
+        state->encoded_block_size = strm->block_size;
+    }
+
+    state->rsip = state->rsi_buffer;
+    state->flush_start = state->rsi_buffer;
+    state->bitp = 0;
+    state->fs = 0;
+    state->mode = m_id;
+
+    rsi_size = strm->rsi * strm->block_size * state->bytes_per_sample;
+    rsi_n = pos / rsi_size;
+    if (rsi_n >= rsi_offsets_count)
+        return AEC_DATA_ERROR;
+
+    /* resize and align to bytes_per_sample */
+    strm_tmp.total_out = 0;
+    strm_tmp.avail_out = size + pos % rsi_size + 1;
+    strm_tmp.avail_out += state->bytes_per_sample - strm_tmp.avail_out % state->bytes_per_sample;
+    if ((out_tmp = malloc(strm_tmp.avail_out)) == NULL)
+        return AEC_MEM_ERROR;
+    strm_tmp.next_out = out_tmp;
+
+    if ((status = aec_buffer_seek(&strm_tmp, rsi_offsets[rsi_n])) != AEC_OK)
+        return status;
+
+    if ((status = aec_decode(&strm_tmp, AEC_FLUSH)) != 0)
+        return status;
+
+    memcpy(strm->next_out, out_tmp + (pos - rsi_n * rsi_size), size);
+
+    strm->next_out += size;
+    strm->avail_out -= size;
+    strm->total_out += size;
+    free(out_tmp);
+
+    return AEC_OK;
+}
+
+int aec_decode_count_offsets(struct aec_stream *strm, size_t *count)
+{
+    struct internal_state *state = strm->state;
+    if (state->offsets == NULL) {
+        *count = 0;
+        return AEC_RSI_OFFSETS_ERROR;
+    } else {
+        *count = vector_size(state->offsets);
+    }
+    return AEC_OK;
+}
+
+int aec_decode_get_offsets(struct aec_stream *strm, size_t *offsets,
+                           size_t offsets_count)
+{
+    struct internal_state *state = strm->state;
+    if (state->offsets == NULL) {
+        return AEC_RSI_OFFSETS_ERROR;
+    }
+    if (offsets_count < vector_size(state->offsets)) {
+        return AEC_MEM_ERROR;
+    }
+    memcpy(offsets, vector_data(state->offsets),
+           vector_size(state->offsets) * sizeof(size_t));
+    return AEC_OK;
+}
+
+int aec_decode_enable_offsets(struct aec_stream *strm)
+{
+    struct internal_state *state = strm->state;
+    if (state->offsets != NULL) {
+        return AEC_RSI_OFFSETS_ERROR;
+    }
+    state->offsets = vector_create();
+    vector_push_back(state->offsets, 0);
+    return AEC_OK;
+}
