@@ -1,20 +1,8 @@
 /**
- * @file decode.c
+ * @file decode.h
  *
  * @section LICENSE
- * Copyright 2012 - 2016
- *
- * Mathis Rosenhauer, Moritz Hanke, Joerg Behrens
- * Deutsches Klimarechenzentrum GmbH
- * Bundesstr. 45a
- * 20146 Hamburg Germany
- *
- * Luis Kornblueh
- * Max-Planck-Institut fuer Meteorologie
- * Bundesstr. 53
- * 20146 Hamburg
- * Germany
- *
+ * Copyright 2026 Mathis Rosenhauer, Moritz Hanke, Joerg Behrens, Luis Kornblueh
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -44,18 +32,17 @@
  * @section DESCRIPTION
  *
  * Adaptive Entropy Decoder
- * Based on CCSDS documents 121.0-B-2 and 120.0-G-3
+ * Based on the CCSDS recommended standard 121.0-B-3
  *
  */
 
 #ifndef DECODE_H
 #define DECODE_H 1
 
-#include <config.h>
-
-#if HAVE_STDINT_H
-#  include <stdint.h>
-#endif
+#include "config.h"
+#include <stdint.h>
+#include <stddef.h>
+#include "vector.h"
 
 #define M_CONTINUE 1
 #define M_EXIT 0
@@ -63,6 +50,10 @@
 
 #define MIN(a, b) (((a) < (b))? (a): (b))
 
+#define SE_TABLE_SIZE 90
+
+struct aec_stream;
+
 struct internal_state {
     int (*mode)(struct aec_stream *);
 
@@ -87,14 +78,13 @@ struct internal_state {
     uint32_t xmax;
 
      /* length of uncompressed input block should be the longest
-        possible block */
+        legal block */
     uint32_t in_blklen;
 
     /* length of output block in bytes */
     uint32_t out_blklen;
 
-    /* counter for samples */
-    uint32_t n, i;
+    uint32_t sample_counter;
 
     /* accumulator for currently used bit sequence */
     uint64_t acc;
@@ -103,11 +93,14 @@ struct internal_state {
     int bitp;
 
     /* last fundamental sequence in accumulator */
-    int fs;
+    uint32_t fs;
 
     /* 1 if current block has reference sample */
     int ref;
 
+    /* block_size minus reference sample if present */
+    uint32_t encoded_block_size;
+
     /* 1 if postprocessor has to be used */
     int pp;
 
@@ -127,7 +120,10 @@ struct internal_state {
     uint32_t *flush_start;
 
     /* table for decoding second extension option */
-    int se_table[182];
-} decode_state;
+    int se_table[2 * (SE_TABLE_SIZE + 1)];
+
+    /* RSI table */
+    struct vector_t *offsets;
+};
 
 #endif /* DECODE_H */
