@@ -2,19 +2,7 @@
  * @file encode.c
  *
  * @section LICENSE
- * Copyright 2012 - 2016
- *
- * Mathis Rosenhauer, Moritz Hanke, Joerg Behrens
- * Deutsches Klimarechenzentrum GmbH
- * Bundesstr. 45a
- * 20146 Hamburg Germany
- *
- * Luis Kornblueh
- * Max-Planck-Institut fuer Meteorologie
- * Bundesstr. 53
- * 20146 Hamburg
- * Germany
- *
+ * Copyright 2026 Mathis Rosenhauer, Moritz Hanke, Joerg Behrens, Luis Kornblueh
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -44,18 +32,18 @@
  * @section DESCRIPTION
  *
  * Adaptive Entropy Encoder
- * Based on CCSDS documents 121.0-B-2 and 120.0-G-3
+ * Based on the CCSDS recommended standard 121.0-B-3
  *
  */
 
+#include "config.h"
+#include "encode.h"
+#include "encode_accessors.h"
+#include "libaec.h"
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 
-#include "libaec.h"
-#include "encode.h"
-#include "encode_accessors.h"
-
 static int m_get_block(struct aec_stream *strm);
 
 static inline void emit(struct internal_state *state,
@@ -117,15 +105,12 @@ static inline void copy64(uint8_t *dst, uint64_t src)
 
 static inline void emitblock_fs(struct aec_stream *strm, int k, int ref)
 {
-    size_t i;
-    uint32_t used; /* used bits in 64 bit accumulator */
-    uint64_t acc; /* accumulator */
     struct internal_state *state = strm->state;
 
-    acc = (uint64_t)*state->cds << 56;
-    used = 7 - state->bits;
+    uint64_t acc = (uint64_t)*state->cds << 56;
+    uint32_t used = 7 - state->bits; /* used bits in 64 bit accumulator */
 
-    for (i = ref; i < strm->block_size; i++) {
+    for (size_t i = ref; i < strm->block_size; i++) {
         used += (state->block[i] >> k) + 1;
         while (used > 63) {
             copy64(state->cds, acc);
@@ -147,16 +132,14 @@ static inline void emitblock(struct aec_stream *strm, int k, int ref)
        Emit the k LSB of a whole block of input data.
     */
 
-    uint64_t a;
     struct internal_state *state = strm->state;
     uint32_t *in = state->block + ref;
     uint32_t *in_end = state->block + strm->block_size;
     uint64_t mask = (UINT64_C(1) << k) - 1;
     uint8_t *o = state->cds;
+    uint64_t a = *o;
     int p = state->bits;
 
-    a = *o;
-
     while(in < in_end) {
         a <<= 56;
         p = (p % 8) + 56;
@@ -238,21 +221,20 @@ static void preprocess_unsigned(struct aec_stream *strm)
        Preprocess RSI of unsigned samples.
 
        Combining preprocessing and converting to uint32_t in one loop
-       is slower due to the data dependance on x_i-1.
+       is slower due to the data dependence on x_i-1.
     */
 
     uint32_t D;
     struct internal_state *state = strm->state;
-    const uint32_t *restrict x = state->data_raw;
-    uint32_t *restrict d = state->data_pp;
+    const uint32_t *x = state->data_raw;
+    uint32_t *d = state->data_pp;
     uint32_t xmax = state->xmax;
     uint32_t rsi = strm->rsi * strm->block_size - 1;
-    size_t i;
 
     state->ref = 1;
     state->ref_sample = x[0];
     d[0] = 0;
-    for (i = 0; i < rsi; i++) {
+    for (size_t i = 0; i < rsi; i++) {
         if (x[i + 1] >= x[i]) {
             D = x[i + 1] - x[i];
             if (D <= x[i])
@@ -278,30 +260,30 @@ static void preprocess_signed(struct aec_stream *strm)
 
     uint32_t D;
     struct internal_state *state = strm->state;
-    int32_t *restrict x = (int32_t *)state->data_raw;
-    uint32_t *restrict d = state->data_pp;
-    int32_t xmax = (int32_t)state->xmax;
-    int32_t xmin = (int32_t)state->xmin;
+    uint32_t *x = state->data_raw;
+    uint32_t *d = state->data_pp;
+    uint32_t xmax = state->xmax;
+    uint32_t xmin = state->xmin;
     uint32_t rsi = strm->rsi * strm->block_size - 1;
-    uint32_t m = UINT64_C(1) << (strm->bits_per_sample - 1);
-    size_t i;
+    uint32_t m = UINT32_C(1) << (strm->bits_per_sample - 1);
 
     state->ref = 1;
     state->ref_sample = x[0];
     d[0] = 0;
+    /* Sign extension */
     x[0] = (x[0] ^ m) - m;
 
-    for (i = 0; i < rsi; i++) {
+    for (size_t i = 0; i < rsi; i++) {
         x[i + 1] = (x[i + 1] ^ m) - m;
-        if (x[i + 1] < x[i]) {
-            D = (uint32_t)(x[i] - x[i + 1]);
-            if (D <= (uint32_t)(xmax - x[i]))
+        if ((int32_t)x[i + 1] < (int32_t)x[i]) {
+            D = x[i] - x[i + 1];
+            if (D <= xmax - x[i])
                 d[i + 1] = 2 * D - 1;
             else
                 d[i + 1] = xmax - x[i + 1];
         } else {
-            D = (uint32_t)(x[i + 1] - x[i]);
-            if (D <= (uint32_t)(x[i] - xmin))
+            D = x[i + 1] - x[i];
+            if (D <= x[i] - xmin)
                 d[i + 1] = 2 * D;
             else
                 d[i + 1] = x[i + 1] - xmin;
@@ -316,11 +298,10 @@ static inline uint64_t block_fs(struct aec_stream *strm, int k)
        Sum FS of all samples in block for given splitting position.
     */
 
-    size_t i;
-    uint64_t fs = 0;
     struct internal_state *state = strm->state;
+    uint64_t fs = 0;
 
-    for (i = 0; i < strm->block_size; i++)
+    for (size_t i = 0; i < strm->block_size; i++)
         fs += (uint64_t)(state->block[i] >> k);
 
     return fs;
@@ -334,7 +315,7 @@ static uint32_t assess_splitting_option(struct aec_stream *strm)
        In Rice coding each sample in a block of samples is split at
        the same position into k LSB and bits_per_sample - k MSB. The
        LSB part is left binary and the MSB part is coded as a
-       fundamental sequence a.k.a. unary (see CCSDS 121.0-B-2). The
+       fundamental sequence a.k.a. unary (see CCSDS 121.0-B-3). The
        function of the length of the Coded Data Set (CDS) depending on
        k has exactly one minimum (see A. Kiely, IPN Progress Report
        42-159).
@@ -346,33 +327,36 @@ static uint32_t assess_splitting_option(struct aec_stream *strm)
        increases, then the minimum will be found with smaller k.
 
        For increasing k we know that we will gain block_size bits in
-       length through the larger binary part. If the FS lenth is less
+       length through the larger binary part. If the FS length is less
        than the block size then a reduced FS part can't compensate the
        larger binary part. So we know that the CDS for k+1 will be
        larger than for k without actually computing the length. An
        analogue check can be done for decreasing k.
      */
 
-    int k;
-    int k_min;
-    int this_bs; /* Block size of current block */
-    int no_turn; /* 1 if we shouldn't reverse */
-    int dir; /* Direction, 1 means increasing k, 0 decreasing k */
-    uint64_t len; /* CDS length for current k */
-    uint64_t len_min; /* CDS length minimum so far */
-    uint64_t fs_len; /* Length of FS part (not including 1s) */
-
     struct internal_state *state = strm->state;
 
-    this_bs = strm->block_size - state->ref;
-    len_min = UINT64_MAX;
-    k = k_min = state->k;
-    no_turn = k == 0;
-    dir = 1;
+    /* Block size of current block */
+    uint64_t this_bs = strm->block_size - state->ref;
+
+    /* CDS length minimum so far */
+    uint64_t len_min = UINT64_MAX;
+
+    int k = state->k;
+    int k_min = k;
+
+    /* 1 if we shouldn't reverse */
+    int no_turn = (k == 0);
+
+    /* Direction, 1 means increasing k, 0 decreasing k */
+    int dir = 1;
 
     for (;;) {
-        fs_len = block_fs(strm, k);
-        len = fs_len + this_bs * (k + 1);
+        /* Length of FS part (not including 1s) */
+        uint64_t fs_len = block_fs(strm, k);
+
+        /* CDS length for current k */
+        uint64_t len = fs_len + this_bs * (k + 1);
 
         if (len < len_min) {
             if (len_min < UINT64_MAX)
@@ -417,19 +401,17 @@ static uint32_t assess_se_option(struct aec_stream *strm)
        If length is above limit just return UINT32_MAX.
     */
 
-    size_t i;
-    uint64_t len, d;
     struct internal_state *state = strm->state;
     uint32_t *block = state->block;
+    uint64_t len = 1;
 
-    len = 1;
-
-    for (i = 0; i < strm->block_size; i += 2) {
-        d = (uint64_t)block[i] + (uint64_t)block[i + 1];
+    for (size_t i = 0; i < strm->block_size; i += 2) {
+        uint64_t d = (uint64_t)block[i] + (uint64_t)block[i + 1];
         len += d * (d + 1) / 2 + block[i + 1] + 1;
         if (len > state->uncomp_len)
             return UINT32_MAX;
     }
+
     return (uint32_t)len;
 }
 
@@ -493,22 +475,28 @@ static int m_flush_block(struct aec_stream *strm)
 
        Fall back to slow flushing if in buffered mode.
     */
-    int n;
     struct internal_state *state = strm->state;
 
 #ifdef ENABLE_RSI_PADDING
-    if (state->blocks_avail == 0
-        && strm->flags & AEC_PAD_RSI
-        && state->block_nonzero == 0
-        )
-        emit(state, 0, state->bits % 8);
+    if (state->blocks_avail == 0 &&
+        strm->flags & AEC_PAD_RSI &&
+        state->block_nonzero == 0)
+            emit(state, 0, state->bits % 8);
 #endif
 
     if (state->direct_out) {
-        n = (int)(state->cds - strm->next_out);
+        int n = (int)(state->cds - strm->next_out);
         strm->next_out += n;
         strm->avail_out -= n;
         state->mode = m_get_block;
+
+        if (state->ready_to_capture_rsi &&
+            state->blocks_avail == 0 &&
+            state->offsets != NULL) {
+                vector_push_back(state->offsets, (strm->total_out - strm->avail_out) * 8 + (8 - state->bits));
+                state->ready_to_capture_rsi = 0;
+        }
+
         return M_CONTINUE;
     }
 
@@ -546,16 +534,14 @@ static int m_encode_uncomp(struct aec_stream *strm)
 
 static int m_encode_se(struct aec_stream *strm)
 {
-    size_t i;
-    uint32_t d;
     struct internal_state *state = strm->state;
 
     emit(state, 1, state->id_len + 1);
     if (state->ref)
         emit(state, state->ref_sample, strm->bits_per_sample);
 
-    for (i = 0; i < strm->block_size; i+= 2) {
-        d = state->block[i] + state->block[i + 1];
+    for (size_t i = 0; i < strm->block_size; i+= 2) {
+        uint32_t d = state->block[i] + state->block[i + 1];
         emitfs(state, d * (d + 1) / 2 + state->block[i + 1]);
     }
 
@@ -588,10 +574,10 @@ static int m_select_code_option(struct aec_stream *strm)
        Decide which code option to use.
     */
 
-    uint32_t split_len;
-    uint32_t se_len;
     struct internal_state *state = strm->state;
 
+    uint32_t split_len;
+    uint32_t se_len;
     if (state->id_len > 1)
         split_len = assess_splitting_option(strm);
     else
@@ -620,10 +606,10 @@ static int m_check_zero_block(struct aec_stream *strm)
        end of a segment or RSI.
     */
 
-    size_t i;
     struct internal_state *state = strm->state;
     uint32_t *p = state->block;
 
+    size_t i;
     for (i = 0; i < strm->block_size; i++)
         if (p[i] != 0)
             break;
@@ -663,7 +649,7 @@ static int m_get_rsi_resumable(struct aec_stream *strm)
     /**
        Get RSI while input buffer is short.
 
-       Let user provide more input. Once we got all input pad buffer
+       Let user provide more input. Once we got all input, pad buffer
        to full RSI.
     */
 
@@ -678,6 +664,8 @@ static int m_get_rsi_resumable(struct aec_stream *strm)
                     state->blocks_avail = state->i / strm->block_size - 1;
                     if (state->i % strm->block_size)
                         state->blocks_avail++;
+                    /* Pad raw buffer with last sample. Only encode
+                     * blocks_avail will be encoded later. */
                     do
                         state->data_raw[state->i] =
                             state->data_raw[state->i - 1];
@@ -729,8 +717,8 @@ static int m_get_block(struct aec_stream *strm)
         state->blocks_avail = strm->rsi - 1;
         state->block = state->data_pp;
         state->blocks_dispensed = 1;
-
         if (strm->avail_in >= state->rsi_len) {
+            state->ready_to_capture_rsi = 1;
             state->get_rsi(strm);
             if (strm->flags & AEC_DATA_PREPROCESS)
                 state->preprocess(strm);
@@ -860,12 +848,12 @@ int aec_encode_init(struct aec_stream *strm)
     state->rsi_len = strm->rsi * strm->block_size * state->bytes_per_sample;
 
     if (strm->flags & AEC_DATA_SIGNED) {
-        state->xmax = UINT32_MAX >> (32 - strm->bits_per_sample + 1);
+        state->xmax = (INT64_C(1) << (strm->bits_per_sample - 1)) - 1;
         state->xmin = ~state->xmax;
         state->preprocess = preprocess_signed;
     } else {
+        state->xmax = (UINT64_C(1) << strm->bits_per_sample) - 1;
         state->xmin = 0;
-        state->xmax = UINT32_MAX >> (32 - strm->bits_per_sample);
         state->preprocess = preprocess_unsigned;
     }
 
@@ -902,7 +890,7 @@ int aec_encode_init(struct aec_stream *strm)
     *state->cds = 0;
     state->bits = 8;
     state->mode = m_get_block;
-
+    state->ready_to_capture_rsi = 0;
     return AEC_OK;
 }
 
@@ -912,7 +900,6 @@ int aec_encode(struct aec_stream *strm, int flush)
        Finite-state machine implementation of the adaptive entropy
        encoder.
     */
-    int n;
     struct internal_state *state = strm->state;
 
     state->flush = flush;
@@ -922,7 +909,7 @@ int aec_encode(struct aec_stream *strm, int flush)
     while (state->mode(strm) == M_CONTINUE);
 
     if (state->direct_out) {
-        n = (int)(state->cds - strm->next_out);
+        int n = (int)(state->cds - strm->next_out);
         strm->next_out += n;
         strm->avail_out -= n;
 
@@ -938,20 +925,58 @@ int aec_encode(struct aec_stream *strm, int flush)
 int aec_encode_end(struct aec_stream *strm)
 {
     struct internal_state *state = strm->state;
-    int status;
 
-    status = AEC_OK;
+    int status = AEC_OK;
     if (state->flush == AEC_FLUSH && state->flushed == 0)
         status = AEC_STREAM_ERROR;
+    if (state->offsets != NULL) {
+        vector_destroy(state->offsets);
+        state->offsets = NULL;
+    }
     cleanup(strm);
     return status;
 }
 
+int aec_encode_count_offsets(struct aec_stream *strm, size_t *count)
+{
+    struct internal_state *state = strm->state;
+    if (state->offsets == NULL) {
+        *count = 0;
+        return AEC_RSI_OFFSETS_ERROR;
+    } else {
+        *count = vector_size(state->offsets);
+    }
+    return AEC_OK;
+}
+
+int aec_encode_get_offsets(struct aec_stream *strm, size_t *offsets, size_t offsets_count)
+{
+    struct internal_state *state = strm->state;
+    if (state->offsets == NULL) {
+        return AEC_RSI_OFFSETS_ERROR;
+    }
+    if (offsets_count < vector_size(state->offsets)) {
+        return AEC_MEM_ERROR;
+    }
+    memcpy(offsets, vector_data(state->offsets), offsets_count * sizeof(size_t));
+    return AEC_OK;
+}
+
+int aec_encode_enable_offsets(struct aec_stream *strm)
+{
+    struct internal_state *state = strm->state;
+
+    if (state->offsets != NULL)
+        return AEC_RSI_OFFSETS_ERROR;
+
+    state->offsets = vector_create();
+    vector_push_back(state->offsets, 0);
+    return AEC_OK;
+}
+
 int aec_buffer_encode(struct aec_stream *strm)
 {
-    int status;
-
-    status = aec_encode_init(strm);
+    int status = aec_encode_init(strm);
     if (status != AEC_OK)
         return status;
     status = aec_encode(strm, AEC_FLUSH);
