@@ -2,19 +2,7 @@
  * @file encode.h
  *
  * @section LICENSE
- * Copyright 2012 - 2016
- *
- * Mathis Rosenhauer, Moritz Hanke, Joerg Behrens
- * Deutsches Klimarechenzentrum GmbH
- * Bundesstr. 45a
- * 20146 Hamburg Germany
- *
- * Luis Kornblueh
- * Max-Planck-Institut fuer Meteorologie
- * Bundesstr. 53
- * 20146 Hamburg
- * Germany
- *
+ * Copyright 2026 Mathis Rosenhauer, Moritz Hanke, Joerg Behrens, Luis Kornblueh
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -44,18 +32,16 @@
  * @section DESCRIPTION
  *
  * Adaptive Entropy Encoder
- * Based on CCSDS documents 121.0-B-2 and 120.0-G-3
+ * Based on the CCSDS recommended standard 121.0-B-3
  *
  */
 
 #ifndef ENCODE_H
 #define ENCODE_H 1
 
-#include <config.h>
-
-#if HAVE_STDINT_H
-#  include <stdint.h>
-#endif
+#include "vector.h"
+#include "config.h"
+#include <stdint.h>
 
 #define M_CONTINUE 1
 #define M_EXIT 0
@@ -68,6 +54,8 @@
 /* Marker for Remainder Of Segment condition in zero block encoding */
 #define ROS -1
 
+struct aec_stream;
+
 struct internal_state {
     int (*mode)(struct aec_stream *);
     uint32_t (*get_sample)(struct aec_stream *);
@@ -153,6 +141,12 @@ struct internal_state {
 
     /* length of uncompressed CDS */
     uint32_t uncomp_len;
+
+    /* RSI offsets container */
+    struct vector_t *offsets;
+
+    /* indicator if an RSI should be captured */
+    int ready_to_capture_rsi;
 };
 
 #endif /* ENCODE_H */
