@@ -2,19 +2,7 @@
  * @file encode_accessors.c
  *
  * @section LICENSE
- * Copyright 2012 - 2016
- *
- * Mathis Rosenhauer, Moritz Hanke, Joerg Behrens
- * Deutsches Klimarechenzentrum GmbH
- * Bundesstr. 45a
- * 20146 Hamburg Germany
- *
- * Luis Kornblueh
- * Max-Planck-Institut fuer Meteorologie
- * Bundesstr. 53
- * 20146 Hamburg
- * Germany
- *
+ * Copyright 2026 Mathis Rosenhauer, Moritz Hanke, Joerg Behrens, Luis Kornblueh
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -47,16 +35,12 @@
  *
  */
 
-#include <config.h>
-
-#if HAVE_STDINT_H
-# include <stdint.h>
-#endif
-
-#include <string.h>
-#include "libaec.h"
-#include "encode.h"
+#include "config.h"
 #include "encode_accessors.h"
+#include "encode.h"
+#include "libaec.h"
+#include <stdint.h>
+#include <string.h>
 
 uint32_t aec_get_8(struct aec_stream *strm)
 {
@@ -66,9 +50,7 @@ uint32_t aec_get_8(struct aec_stream *strm)
 
 uint32_t aec_get_lsb_16(struct aec_stream *strm)
 {
-    uint32_t data;
-
-    data = ((uint32_t)strm->next_in[1] << 8)
+    uint32_t data = ((uint32_t)strm->next_in[1] << 8)
         | (uint32_t)strm->next_in[0];
 
     strm->next_in += 2;
@@ -78,9 +60,7 @@ uint32_t aec_get_lsb_16(struct aec_stream *strm)
 
 uint32_t aec_get_msb_16(struct aec_stream *strm)
 {
-    uint32_t data;
-
-    data = ((uint32_t)strm->next_in[0] << 8)
+    uint32_t data = ((uint32_t)strm->next_in[0] << 8)
         | (uint32_t)strm->next_in[1];
 
     strm->next_in += 2;
@@ -90,9 +70,7 @@ uint32_t aec_get_msb_16(struct aec_stream *strm)
 
 uint32_t aec_get_lsb_24(struct aec_stream *strm)
 {
-    uint32_t data;
-
-    data = ((uint32_t)strm->next_in[2] << 16)
+    uint32_t data = ((uint32_t)strm->next_in[2] << 16)
         | ((uint32_t)strm->next_in[1] << 8)
         | (uint32_t)strm->next_in[0];
 
@@ -103,9 +81,7 @@ uint32_t aec_get_lsb_24(struct aec_stream *strm)
 
 uint32_t aec_get_msb_24(struct aec_stream *strm)
 {
-    uint32_t data;
-
-    data = ((uint32_t)strm->next_in[0] << 16)
+    uint32_t data = ((uint32_t)strm->next_in[0] << 16)
         | ((uint32_t)strm->next_in[1] << 8)
         | (uint32_t)strm->next_in[2];
 
@@ -116,9 +92,7 @@ uint32_t aec_get_msb_24(struct aec_stream *strm)
 
 uint32_t aec_get_lsb_32(struct aec_stream *strm)
 {
-    uint32_t data;
-
-    data = ((uint32_t)strm->next_in[3] << 24)
+    uint32_t data = ((uint32_t)strm->next_in[3] << 24)
         | ((uint32_t)strm->next_in[2] << 16)
         | ((uint32_t)strm->next_in[1] << 8)
         | (uint32_t)strm->next_in[0];
@@ -130,9 +104,7 @@ uint32_t aec_get_lsb_32(struct aec_stream *strm)
 
 uint32_t aec_get_msb_32(struct aec_stream *strm)
 {
-    uint32_t data;
-
-    data = ((uint32_t)strm->next_in[0] << 24)
+    uint32_t data = ((uint32_t)strm->next_in[0] << 24)
         | ((uint32_t)strm->next_in[1] << 16)
         | ((uint32_t)strm->next_in[2] << 8)
         | (uint32_t)strm->next_in[3];
@@ -144,12 +116,11 @@ uint32_t aec_get_msb_32(struct aec_stream *strm)
 
 void aec_get_rsi_8(struct aec_stream *strm)
 {
-    int i;
-    uint32_t *restrict out = strm->state->data_raw;
-    unsigned const char *restrict in = strm->next_in;
-    int rsi = strm->rsi * strm->block_size;
+    uint32_t *out = strm->state->data_raw;
+    unsigned const char *in = strm->next_in;
+    int rsi = strm->rsi *strm->block_size;
 
-    for (i = 0; i < rsi; i++)
+    for (int i = 0; i < rsi; i++)
         out[i] = (uint32_t)in[i];
 
     strm->next_in += rsi;
@@ -158,12 +129,11 @@ void aec_get_rsi_8(struct aec_stream *strm)
 
 void aec_get_rsi_lsb_16(struct aec_stream *strm)
 {
-    int i;
-    uint32_t *restrict out = strm->state->data_raw;
-    const unsigned char *restrict in = strm->next_in;
-    int rsi = strm->rsi * strm->block_size;
+    uint32_t *out = strm->state->data_raw;
+    const unsigned char *in = strm->next_in;
+    int rsi = strm->rsi *strm->block_size;
 
-    for (i = 0; i < rsi; i++)
+    for (int i = 0; i < rsi; i++)
         out[i] = (uint32_t)in[2 * i] | ((uint32_t)in[2 * i + 1] << 8);
 
     strm->next_in += 2 * rsi;
@@ -172,12 +142,11 @@ void aec_get_rsi_lsb_16(struct aec_stream *strm)
 
 void aec_get_rsi_msb_16(struct aec_stream *strm)
 {
-    int i;
-    uint32_t *restrict out = strm->state->data_raw;
-    const unsigned char *restrict in = strm->next_in;
-    int rsi = strm->rsi * strm->block_size;
+    uint32_t *out = strm->state->data_raw;
+    const unsigned char *in = strm->next_in;
+    int rsi = strm->rsi *strm->block_size;
 
-    for (i = 0; i < rsi; i++)
+    for (int i = 0; i < rsi; i++)
         out[i] = ((uint32_t)in[2 * i] << 8) | (uint32_t)in[2 * i + 1];
 
     strm->next_in += 2 * rsi;
@@ -186,12 +155,11 @@ void aec_get_rsi_msb_16(struct aec_stream *strm)
 
 void aec_get_rsi_lsb_24(struct aec_stream *strm)
 {
-    int i;
-    uint32_t *restrict out = strm->state->data_raw;
-    const unsigned char *restrict in = strm->next_in;
-    int rsi = strm->rsi * strm->block_size;
+    uint32_t *out = strm->state->data_raw;
+    const unsigned char *in = strm->next_in;
+    int rsi = strm->rsi *strm->block_size;
 
-    for (i = 0; i < rsi; i++)
+    for (int i = 0; i < rsi; i++)
         out[i] = (uint32_t)in[3 * i]
             | ((uint32_t)in[3 * i + 1] << 8)
             | ((uint32_t)in[3 * i + 2] << 16);
@@ -202,12 +170,11 @@ void aec_get_rsi_lsb_24(struct aec_stream *strm)
 
 void aec_get_rsi_msb_24(struct aec_stream *strm)
 {
-    int i;
-    uint32_t *restrict out = strm->state->data_raw;
-    const unsigned char *restrict in = strm->next_in;
-    int rsi = strm->rsi * strm->block_size;
+    uint32_t *out = strm->state->data_raw;
+    const unsigned char *in = strm->next_in;
+    int rsi = strm->rsi *strm->block_size;
 
-    for (i = 0; i < rsi; i++)
+    for (int i = 0; i < rsi; i++)
         out[i] = ((uint32_t)in[3 * i] << 16)
             | ((uint32_t)in[3 * i + 1] << 8)
             | (uint32_t)in[3 * i + 2];
@@ -219,7 +186,7 @@ void aec_get_rsi_msb_24(struct aec_stream *strm)
 #define AEC_GET_RSI_NATIVE_32(BO)                       \
     void aec_get_rsi_##BO##_32(struct aec_stream *strm) \
     {                                               \
-        int rsi = strm->rsi * strm->block_size;     \
+        int rsi = strm->rsi *strm->block_size;     \
         memcpy(strm->state->data_raw,               \
                strm->next_in, 4 * rsi);             \
         strm->next_in += 4 * rsi;                   \
@@ -229,12 +196,11 @@ void aec_get_rsi_msb_24(struct aec_stream *strm)
 #ifdef WORDS_BIGENDIAN
 void aec_get_rsi_lsb_32(struct aec_stream *strm)
 {
-    int i;
-    uint32_t *restrict out = strm->state->data_raw;
-    const unsigned char *restrict in = strm->next_in;
-    int rsi = strm->rsi * strm->block_size;
+    uint32_t *out = strm->state->data_raw;
+    const unsigned char *in = strm->next_in;
+    int rsi = strm->rsi *strm->block_size;
 
-    for (i = 0; i < rsi; i++)
+    for (int i = 0; i < rsi; i++)
         out[i] = (uint32_t)in[4 * i]
             | ((uint32_t)in[4 * i + 1] << 8)
             | ((uint32_t)in[4 * i + 2] << 16)
@@ -249,15 +215,14 @@ AEC_GET_RSI_NATIVE_32(msb);
 #else /* !WORDS_BIGENDIAN */
 void aec_get_rsi_msb_32(struct aec_stream *strm)
 {
-    int i;
-    uint32_t *restrict out = strm->state->data_raw;
-    const unsigned char *restrict in = strm->next_in;
-    int rsi = strm->rsi * strm->block_size;
+    uint32_t *out = strm->state->data_raw;
+    const unsigned char *in = strm->next_in;
+    int rsi = strm->rsi *strm->block_size;
 
     strm->next_in += 4 * rsi;
     strm->avail_in -= 4 * rsi;
 
-    for (i = 0; i < rsi; i++)
+    for (int i = 0; i < rsi; i++)
         out[i] = ((uint32_t)in[4 * i] << 24)
             | ((uint32_t)in[4 * i + 1] << 16)
             | ((uint32_t)in[4 * i + 2] << 8)
