@@ -4,19 +4,7 @@
  * @author Thomas Jahns, Deutsches Klimarechenzentrum
  *
  * @section LICENSE
- * Copyright 2012 - 2016
- *
- * Mathis Rosenhauer, Moritz Hanke, Joerg Behrens
- * Deutsches Klimarechenzentrum GmbH
- * Bundesstr. 45a
- * 20146 Hamburg Germany
- *
- * Luis Kornblueh
- * Max-Planck-Institut fuer Meteorologie
- * Bundesstr. 53
- * 20146 Hamburg
- * Germany
- *
+ * Copyright 2026 Mathis Rosenhauer, Moritz Hanke, Joerg Behrens, Luis Kornblueh
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -48,10 +36,6 @@
  * Simple timing command, since calling time(1) gives non-portable results.
  *
  */
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
-
 #include <errno.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -67,7 +51,8 @@ run_cmd(int argc, char *argv[]);
 int main(int argc, char **argv)
 {
   struct timeval utime = { .tv_sec = 0, .tv_usec = 0 };
-  int status, rstatus;
+  int rstatus;
+  int status = 0;
   if (argc > 1 && ((status = run_cmd(argc - 1, argv + 1)) >= 0))
   {
     struct rusage usage;
@@ -120,4 +105,3 @@ run_cmd(int argc, char *argv[])
   }
   return status;
 }
-
